# AllQuiet (c) 2007 by Peter Watkins (peterw@tux.org) 
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, which should be included with this software.

package Plugins::AllQuiet::Plugin;

use strict;

use Slim::Control::Command;
use Slim::Control::Request;
use Slim::Utils::DateTime;
use Slim::Utils::Strings qw (string);
use Slim::Utils::Misc qw( msg );
use POSIX qw(strftime);

$Plugins::AllQuiet::Plugin::apiVersion = 1.0;

# ------------------- variables -----------------------
my $displayDuration = 30;	# how long to show message on players that are "off" or idle
# ------------------- variables -----------------------

my $pausedBy = undef;	# name of client that initiated pausing
my %brightness;		# to revert brightness level

use vars qw($VERSION);
$VERSION = &vlVersion();

# our i18n/l10n strings
sub strings() { 
  	return '
PLUGIN_ALLQUIET_NOTIFICATION
	EN	Paused by 

PLUGIN_ALLQUIET
	EN	AllQuiet
'};

# whether we're enabled (the callback remains in the function 
# stack after "disabling" the plugin, so we need to keep track of this)
my $pluginEnabled = 0;

# setup routine
sub initPlugin {
	if ( $pluginEnabled == 0 ) {
		# register a CLI command
		Slim::Control::Request::addDispatch(['allquiet','_message'], [0, 0, 0, \&Plugins::AllQuiet::Plugin::allquietCLI]);
		# use Slim::Web::HTTP::protectCommand to protect against CSRF
		if ( defined(&Slim::Web::HTTP::protectCommand) ) {
			&Plugin_msg("protecting 'allquiet' command against CSRF attacks\n");
			Slim::Web::HTTP::protectCommand('allquiet');
		} else {
			&Plugin_msg("Warning: unable to protect 'allquiet' command from CSRF attacks\n");
		}
	}
	# note that we should act
	$pluginEnabled = 1;
	$::d_plugins && &Plugin_msg("enabled\n");
}

sub shutdownPlugin {
	# we should not act
	$pluginEnabled = 0;
	$::d_plugins && &Plugin_msg("disabled\n");
}

sub enabled {
	if ( substr($::VERSION,0,3) gt '6.5' ) {
		&Plugin_msg("This plugin has not been tested with SlimServer newer than 6.5.x\n");
	}
	return ($::VERSION ge '6.5');
}

sub vlVersion() {
	my $rcsVersion = '$Revision: 1.1 $';
	$rcsVersion =~ s/.*:\s*([0-9\.]*).*$/$1/;
	return $rcsVersion;
}

sub getDisplayName() {
	return 'PLUGIN_ALLQUIET';
}

sub allquietCLI {
	my $request = shift;
	# much of this from IR Blaster
        # Check this is the correct query
        if( $request->isNotCommand( [['allquiet']])) {
		$request->setStatusBadDispatch();
		return;
	}
	my $message = $request->getParam( '_message');
	if (! defined($message) ) {
		$request->setStatusBadDispatch();
		return;
	}
	$pausedBy = $message;
	&Plugin_msg("called to silence players by CLI, message \"$message\"\n");
	&allquiet(undef);
}

# function that does the work
sub allquiet {
	my $client = shift;
	if ( $pluginEnabled == 0 ) {
		&Plugin_msg("AllQuiet ignoring button press (plugin disabled)\n");
		return;
	}
	if ( defined($client) ) {
		my $id = $client->id();
		my $name = Slim::Player::Client::name($client);
		$pausedBy = string('PLUGIN_ALLQUIET_NOTIFICATION') . $name;
		&Plugin_msg("called to silence players by player \"$name\"\n");
	}
	# get a list of all the players
	my @players = Slim::Player::Client::clients();
	# consider them one by one
	for my $player (@players) {
		my $pid = $player->id();
		my $mode = Slim::Player::Source::playmode($player);
		if ( $mode eq 'play' ) {
			# pause player
			Slim::Control::Request::executeRequest($player, ['pause', '_newvalue', 1], undef, undef);
			# push into AllQuiet mode so user is most likley to see the message
			Slim::Buttons::Common::pushModeLeft($player, 'PLUGIN.AllQuiet::Plugin');
			# set timer to restore brightness after $displayDuration seconds
			Slim::Utils::Timers::killTimers($player, \&brightnessReset);
			Slim::Utils::Timers::setTimer($player, (Time::HiRes::time() + $displayDuration), \&brightnessReset);
		} else {
			# use showBriefly(), especially if player is "off", as offsaver takes over too quickly; 
			my ($line1, $line2) = &displayLines;
			# my ($parsed, $duration, $firstLine, $blockUpdate, $scrollToEnd, $brightness, $callback, $callbackargs, $name);
			$player->showBriefly($line1,$line2, $displayDuration, undef, undef, undef, 4);
		}
	}
}

sub Plugin_msg {
	my $message = shift;
	msg("AllQuiet: $message");
}

# player UI code

sub setMode {
        my $client = shift;
	my $id = $client->id();
	if (! defined($brightness{$id}) ) { 
		$brightness{$id} = $client->brightness(); 
	}
	$client->display->brightness(4);
	$client->lines(\&displayLines);
}

sub brightnessReset {
	my $client = shift;
	my $id = $client->id();
	if (defined($brightness{$id}) ) { 
		$client->display->brightness($brightness{$id});
		$brightness{$id} = undef;
	}
}

our %menuFunctions = (
        # normal up/right/left/down navigation:
        # all we offer is a way to exit this mode
        'left' => sub  {
                my $client = shift;
		my $button = shift;
		my $id = $client->id();
		if (defined($brightness{$id}) ) { 
			$client->display->brightness($brightness{$id});
			$brightness{$id} = undef;
		}
		Slim::Utils::Timers::killTimers($client, \&brightnessReset);
                Slim::Buttons::Common::popModeRight($client);
        },
	# and the function that does the work
	'allquiet' => sub {
                my $client = shift;
		&allquiet($client);
	},
);

sub displayLines {
	my $client = shift;
	my $line1 = string('PLUGIN_ALLQUIET');
	my $line2 = $pausedBy;
	if (! defined($pausedBy) ) {
		$line2 = string('PLUGIN_ALLQUIET_NOTIFICATION') . string('UNK');
	}
        return($line1,$pausedBy);
}

sub getFunctions {
        return \%menuFunctions;
}

1;

